/*
 * $Id: CollapsiblePane.java,v 1.0, rawak.de $
 *
 * Copyright (c) 2001-2011 by the authors of ImageJ2x
 * All rights reserved.
 *
 *
 */
package ij2x.plugin;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Image;
import java.awt.Insets;
import java.awt.Toolkit;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;
import java.net.URL;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

import java.util.MissingResourceException;
import java.util.Locale;
import java.util.ResourceBundle;

import javax.swing.AbstractButton;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.ImageIcon;
import javax.swing.Icon;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.SwingConstants;
import javax.swing.BorderFactory;
import javax.swing.border.Border;
import javax.swing.border.BevelBorder;

import javax.swing.JPopupMenu;
import java.util.Hashtable;
import javax.swing.Action;

import ij.IJ;
import ij.ImageJ;
import ij.plugin.PlugIn;
import ij.Prefs;

import ij.WindowManager;
import ij.ImagePlus;


    public class CollapsiblePane extends JPanel implements PlugIn,
      PropertyChangeListener,
          ActionListener,
              MouseListener,
                    MouseMotionListener{

    protected static ResourceBundle resources;
    protected ImagePlus imp;
    protected boolean systemLF;
    protected Icon icon;
    protected BevelBorder bb ;//new BevelBorder for LOWER or RAISED type
    protected Border grayBorder = BorderFactory.createBevelBorder(bb.RAISED, Color.gray, Color.lightGray);
    protected Border lightGrayBorder = BorderFactory.createBevelBorder(bb.LOWERED, Color.lightGray, Color.lightGray);
    protected JPanel[] commandPanel;
    protected JButton leftButton, rightButton, parentButton;
    public static JButton toolboxButton;


    protected BorderLayout layout;
    private String imageDir = "stIcon";
    private static String[] fileType = {".gif",".png"};
    private String[][] imgName;
    private String[] barName;
    private static String dir;
    protected ImageJ ij;
    private JPanel collapsiblePane;
    private JLabel nextCollapsLabel, parentCollapsLabel;
    private int offset = 80;
    private int offsetHeight;
    private JPopupMenu pm;
    private static Hashtable<Object, Action> barTable;
    private static Action latestAction;

    static {
        try {
            resources = ResourceBundle.getBundle("ij2x.plugin.frame.resources.CollapsiblePane",
                                                 Locale.getDefault());
        } catch (MissingResourceException mre) {
            System.err.println("resources/CollapsiblePane.properties not found");
            System.exit(1);
        }
    }

    public CollapsiblePane() {
        ij = IJ.getInstance();
    }

    public void run(String arg){
        ij = IJ.getInstance();
        createCollapsibleGUI();
    }


    public JPanel createCollapsibleGUI(){

        collapsiblePane = new JPanel();
        nextCollapsLabel=new JLabel("", JLabel.CENTER);
        parentCollapsLabel=new JLabel("", JLabel.CENTER);
        parentCollapsLabel.addMouseListener(this);
        collapsiblePane.addMouseMotionListener(this);

        dir = Prefs.getString(imageDir);

        imgName = new String[1][3];

        imgName[0][0]= "s_toolbar";
        imgName[0][1]= "s_toolbar_ro";
        imgName[0][2]= "s_toolbar_press";

        barName = new String[1];

        barName[0]="Command Bar";


        commandPanel=new JPanel[1];

        commandPanel[0]=new ControlBarPanel();
        commandPanel[0]=((ControlBarPanel)commandPanel[0]).createControlBar();

        toolboxButton=createButton(toolboxButton, "Tool Box", imgName[0], 0);

        this.addPropertyChangeListener(this);

        collapsiblePane.setLayout(new GridBagLayout());

        nextCollapsLabel.setText("ToolBox");
        nextCollapsLabel.setEnabled(false);
        offsetHeight=(int)toolboxButton.getPreferredSize().getHeight();

        if(IJ.isMacintosh()){offset=90;}
        parentCollapsLabel.setMinimumSize(new Dimension(offset, offsetHeight));
        parentCollapsLabel.setMaximumSize(new Dimension(offset, offsetHeight));
        parentCollapsLabel.setPreferredSize(new Dimension(offset, offsetHeight));
        parentCollapsLabel.setText("Command Bar");

            parentCollapsLabel.setToolTipText("Command Bar");
            collapsiblePane.add(parentCollapsLabel,
                                new GridBagConstraints(0, 0, 1, 1, 0.5, 0.0
                    , GridBagConstraints.WEST, GridBagConstraints.NONE,
                    new Insets(0, 0, 0, 0), 0, 0));
            collapsiblePane.add(commandPanel[0],
                                new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0
                    , GridBagConstraints.CENTER, GridBagConstraints.NONE,
                    new Insets(0, 0, 0, 0), 0, 0));
            collapsiblePane.add(toolboxButton,
                                new GridBagConstraints(2, 0, 1, 1, 0.0, 0.0
                    , GridBagConstraints.EAST, GridBagConstraints.NONE,
                    new Insets(0, 0, 0, 0), 0, 0));

        return collapsiblePane;
    }


    private JButton createButton(JButton action, String tooltip, String[] iconName, int i){
        action = new JButton();
        action.setText("");
        action.setToolTipText(tooltip);
        action.addActionListener(this);
        action.addMouseMotionListener(this);
        setIcon(action, iconName, i);
        setBorderlessButton(action);
    return action;
    }

     void setBorderlessButton(JButton button){
         button.setMargin(new Insets(0, 0, 0, 0));
         button.setBorderPainted(false);
         button.setContentAreaFilled(false);
         button.setFocusPainted(false);
      }

      private Icon createIcon(String key, int i){
          Icon icon = null;
          String fileName=key + getFileType(i);
          // if dir is plugin
          if(getImageDir() == null){
           dir="images/";
          }
          String path = getImageDir()+fileName;
         try {
             URL url = this.getClass().getResource(path);
             Image image = Toolkit.getDefaultToolkit().createImage(url);
             if (url != null) {
                 icon = new ImageIcon(image);
               }
             } catch (Exception ex) {
           }
             return icon;
      }


      void setIcon(JButton action, String[] iconName, int j){
          for(int i=0; i < iconName.length; i++){
              if(i == 0){
                  action.setIcon(createIcon(iconName[i], j));
              }
                else if(i == 1){
                    action.setRolloverIcon(createIcon(iconName[i], j));
                }
              else if(i == 2){
                  action.setPressedIcon(createIcon(iconName[i], j));
              }
          }
      }

      private String getFileType(int i){
             return fileType[i];
      }
      private String getImageDir(){
             return dir;
      }

    // Action listener method
    public void actionPerformed(ActionEvent e){
        imp = WindowManager.getCurrentImage();
        //System.out.println("icount="+WindowManager.getImageCount());
        if (imp==null){
        }
           parentButton=(JButton)e.getSource();

           if(parentButton.equals(toolboxButton)){
               ImageJ ij = IJ.getInstance();
               ij.showToolBox();
               return;
           }
   }

    // Actionevent methods Mousemotionlistener
    public void mouseDragged(MouseEvent e) {}
    public void mouseMoved(MouseEvent e) {
    }
    // PropertyChangeEvent method propertyChange
    public void propertyChange(PropertyChangeEvent e){
    }
    public void mousePressed (MouseEvent e) {
            int x=e.getX(), y=e.getY();
            if (e.isPopupTrigger() || e.isMetaDown()){
            }
    }
    public void mouseReleased (MouseEvent e) {}
    public void mouseClicked (MouseEvent e) {}
    public void mouseEntered (MouseEvent e) {}
    public void mouseExited (MouseEvent e) {}



}
